
exports.up = function (knex, Promise) {
    return knex.schema.createTable('group_coauthor_event', function (table) {
        table.inherits('requests');
        table.string('request_from',128).notNullable()
            .references('gid').inTable('groups' /*'simple_groups'*/)
            .onDelete('CASCADE'); //if requesting group is deleted, also delete request
        /*
        ^ as stated in https://www.postgresql.org/docs/10/static/ddl-inherit.html#DDL-INHERIT-CAVEATS,
        "A serious limitation of the inheritance feature is that indexes (including unique constraints) and foreign key constraints only apply to single tables, not to their inheritance children."
        it is not possible (with PostgresQL v10) to use both inheritance and foreign keys at once.
        until it is, or a workaround is found, we simply don't maintain child-level referential integrity
        and we hope all the rest will work, e.g. simple_group_gids will indeed refer to simple groups, and idem for meta_group_gids.
        TODO: maybe, check if PostgresQL's latest version enables this feature?
        */
        table.integer('for_event').notNullable()
            .references('mid').inTable('messages' /*'events'*/)
            .onDelete('CASCADE'); //if event is deleted, also delete request
    });
};

exports.down = function (knex, Promise) {
    return knex.schema.dropTable('group_coauthor_event');
};
